from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error import Error
from ...models.res_group_list import ResGroupList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    creator: Union[Unset, str] = UNSET,
    project: Union[Unset, str] = UNSET,
    x_api_token: str,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["X-Api-Token"] = x_api_token

    params: Dict[str, Any] = {}

    params["creator"] = creator

    params["project"] = project

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/resgroups",
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Error, ResGroupList]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ResGroupList.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Error.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Error.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Error.from_dict(response.json())

        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Error, ResGroupList]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    creator: Union[Unset, str] = UNSET,
    project: Union[Unset, str] = UNSET,
    x_api_token: str,
) -> Response[Union[Error, ResGroupList]]:
    """Get reservation group list

    Args:
        creator (Union[Unset, str]):
        project (Union[Unset, str]):
        x_api_token (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, ResGroupList]]
    """

    kwargs = _get_kwargs(
        creator=creator,
        project=project,
        x_api_token=x_api_token,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    creator: Union[Unset, str] = UNSET,
    project: Union[Unset, str] = UNSET,
    x_api_token: str,
) -> Optional[Union[Error, ResGroupList]]:
    """Get reservation group list

    Args:
        creator (Union[Unset, str]):
        project (Union[Unset, str]):
        x_api_token (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, ResGroupList]
    """

    return sync_detailed(
        client=client,
        creator=creator,
        project=project,
        x_api_token=x_api_token,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    creator: Union[Unset, str] = UNSET,
    project: Union[Unset, str] = UNSET,
    x_api_token: str,
) -> Response[Union[Error, ResGroupList]]:
    """Get reservation group list

    Args:
        creator (Union[Unset, str]):
        project (Union[Unset, str]):
        x_api_token (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, ResGroupList]]
    """

    kwargs = _get_kwargs(
        creator=creator,
        project=project,
        x_api_token=x_api_token,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    creator: Union[Unset, str] = UNSET,
    project: Union[Unset, str] = UNSET,
    x_api_token: str,
) -> Optional[Union[Error, ResGroupList]]:
    """Get reservation group list

    Args:
        creator (Union[Unset, str]):
        project (Union[Unset, str]):
        x_api_token (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, ResGroupList]
    """

    return (
        await asyncio_detailed(
            client=client,
            creator=creator,
            project=project,
            x_api_token=x_api_token,
        )
    ).parsed
