import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.res_group_range import ResGroupRange
    from ..models.res_group_reservation import ResGroupReservation
    from ..models.res_group_route import ResGroupRoute


T = TypeVar("T", bound="ResGroup")


@_attrs_define
class ResGroup:
    """A cloudlab reservation group

    Attributes:
        project (str): The project of the reservation group
        expires_at (datetime.datetime): Expiration time.
        id (Union[Unset, str]): Unique identifier of the reservation group
        reason (Union[Unset, str]): Pithy description of why you want to reserve these resources
        group (Union[Unset, str]): The project subgroup of the reservation group
        creator (Union[Unset, str]): The name of the creator.
        created_at (Union[None, Unset, datetime.datetime]): Creation time.
        start_at (Union[None, Unset, datetime.datetime]): Last modification time.
        powder_zones (Union[None, Unset, str]): Powder only; specify the powder zones when creating range only
            reservation group
        clusters (Union[Unset, List['ResGroupReservation']]):
        ranges (Union[Unset, List['ResGroupRange']]):
        routes (Union[Unset, List['ResGroupRoute']]):
    """

    project: str
    expires_at: datetime.datetime
    id: Union[Unset, str] = UNSET
    reason: Union[Unset, str] = UNSET
    group: Union[Unset, str] = UNSET
    creator: Union[Unset, str] = UNSET
    created_at: Union[None, Unset, datetime.datetime] = UNSET
    start_at: Union[None, Unset, datetime.datetime] = UNSET
    powder_zones: Union[None, Unset, str] = UNSET
    clusters: Union[Unset, List["ResGroupReservation"]] = UNSET
    ranges: Union[Unset, List["ResGroupRange"]] = UNSET
    routes: Union[Unset, List["ResGroupRoute"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
    _immutable_on = {
        "id": {"put": True, "post": True},
        "reason": {"put": True, "post": None},
        "project": {"put": True, "post": None},
        "group": {"put": True, "post": None},
        "creator": {"put": True, "post": True},
        "created_at": {"put": True, "post": True},
        "start_at": {"put": True, "post": None},
        "expires_at": {"put": True, "post": None},
        "powder_zones": {"put": True, "post": None},
        "clusters": {"put": None, "post": None},
        "ranges": {"put": None, "post": None},
        "routes": {"put": None, "post": None},
    }

    def to_dict(self) -> Dict[str, Any]:
        project = self.project

        expires_at = self.expires_at.isoformat()

        id = self.id

        reason = self.reason

        group = self.group

        creator = self.creator

        created_at: Union[None, Unset, str]
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        elif isinstance(self.created_at, datetime.datetime):
            created_at = self.created_at.isoformat()
        else:
            created_at = self.created_at

        start_at: Union[None, Unset, str]
        if isinstance(self.start_at, Unset):
            start_at = UNSET
        elif isinstance(self.start_at, datetime.datetime):
            start_at = self.start_at.isoformat()
        else:
            start_at = self.start_at

        powder_zones: Union[None, Unset, str]
        if isinstance(self.powder_zones, Unset):
            powder_zones = UNSET
        else:
            powder_zones = self.powder_zones

        clusters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.clusters, Unset):
            clusters = []
            for componentsschemas_res_group_reservation_list_item_data in self.clusters:
                componentsschemas_res_group_reservation_list_item = (
                    componentsschemas_res_group_reservation_list_item_data.to_dict()
                )
                clusters.append(componentsschemas_res_group_reservation_list_item)

        ranges: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.ranges, Unset):
            ranges = []
            for componentsschemas_res_group_range_list_item_data in self.ranges:
                componentsschemas_res_group_range_list_item = (
                    componentsschemas_res_group_range_list_item_data.to_dict()
                )
                ranges.append(componentsschemas_res_group_range_list_item)

        routes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.routes, Unset):
            routes = []
            for componentsschemas_res_group_route_list_item_data in self.routes:
                componentsschemas_res_group_route_list_item = (
                    componentsschemas_res_group_route_list_item_data.to_dict()
                )
                routes.append(componentsschemas_res_group_route_list_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project": project,
                "expires_at": expires_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if reason is not UNSET:
            field_dict["reason"] = reason
        if group is not UNSET:
            field_dict["group"] = group
        if creator is not UNSET:
            field_dict["creator"] = creator
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if start_at is not UNSET:
            field_dict["start_at"] = start_at
        if powder_zones is not UNSET:
            field_dict["powder_zones"] = powder_zones
        if clusters is not UNSET:
            field_dict["clusters"] = clusters
        if ranges is not UNSET:
            field_dict["ranges"] = ranges
        if routes is not UNSET:
            field_dict["routes"] = routes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.res_group_range import ResGroupRange
        from ..models.res_group_reservation import ResGroupReservation
        from ..models.res_group_route import ResGroupRoute

        d = src_dict.copy()
        project = d.pop("project")

        expires_at = isoparse(d.pop("expires_at"))

        id = d.pop("id", UNSET)

        reason = d.pop("reason", UNSET)

        group = d.pop("group", UNSET)

        creator = d.pop("creator", UNSET)

        def _parse_created_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                created_at_type_0 = isoparse(data)

                return created_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        created_at = _parse_created_at(d.pop("created_at", UNSET))

        def _parse_start_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_at_type_0 = isoparse(data)

                return start_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        start_at = _parse_start_at(d.pop("start_at", UNSET))

        def _parse_powder_zones(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        powder_zones = _parse_powder_zones(d.pop("powder_zones", UNSET))

        clusters = []
        _clusters = d.pop("clusters", UNSET)
        for componentsschemas_res_group_reservation_list_item_data in _clusters or []:
            componentsschemas_res_group_reservation_list_item = (
                ResGroupReservation.from_dict(
                    componentsschemas_res_group_reservation_list_item_data
                )
            )

            clusters.append(componentsschemas_res_group_reservation_list_item)

        ranges = []
        _ranges = d.pop("ranges", UNSET)
        for componentsschemas_res_group_range_list_item_data in _ranges or []:
            componentsschemas_res_group_range_list_item = ResGroupRange.from_dict(
                componentsschemas_res_group_range_list_item_data
            )

            ranges.append(componentsschemas_res_group_range_list_item)

        routes = []
        _routes = d.pop("routes", UNSET)
        for componentsschemas_res_group_route_list_item_data in _routes or []:
            componentsschemas_res_group_route_list_item = ResGroupRoute.from_dict(
                componentsschemas_res_group_route_list_item_data
            )

            routes.append(componentsschemas_res_group_route_list_item)

        res_group = cls(
            project=project,
            expires_at=expires_at,
            id=id,
            reason=reason,
            group=group,
            creator=creator,
            created_at=created_at,
            start_at=start_at,
            powder_zones=powder_zones,
            clusters=clusters,
            ranges=ranges,
            routes=routes,
        )

        res_group.additional_properties = d
        return res_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
