from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.res_group_range import ResGroupRange
    from ..models.res_group_reservation import ResGroupReservation
    from ..models.res_group_route import ResGroupRoute


T = TypeVar("T", bound="ResGroupError")


@_attrs_define
class ResGroupError:
    """Errors returned when creating/modifying a reservation group

    Attributes:
        error (Union[None, Unset, str]): Error message regarding this reservation group
        error_code (Union[None, Unset, int]): Error code regarding this reservation group
        clusters (Union[Unset, List['ResGroupReservation']]):
        ranges (Union[Unset, List['ResGroupRange']]):
        routes (Union[Unset, List['ResGroupRoute']]):
    """

    error: Union[None, Unset, str] = UNSET
    error_code: Union[None, Unset, int] = UNSET
    clusters: Union[Unset, List["ResGroupReservation"]] = UNSET
    ranges: Union[Unset, List["ResGroupRange"]] = UNSET
    routes: Union[Unset, List["ResGroupRoute"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
    _immutable_on = {
        "error": {"put": True, "post": True},
        "errorCode": {"put": True, "post": True},
        "clusters": {"put": None, "post": None},
        "ranges": {"put": None, "post": None},
        "routes": {"put": None, "post": None},
    }

    def to_dict(self) -> Dict[str, Any]:
        error: Union[None, Unset, str]
        if isinstance(self.error, Unset):
            error = UNSET
        else:
            error = self.error

        error_code: Union[None, Unset, int]
        if isinstance(self.error_code, Unset):
            error_code = UNSET
        else:
            error_code = self.error_code

        clusters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.clusters, Unset):
            clusters = []
            for componentsschemas_res_group_reservation_list_item_data in self.clusters:
                componentsschemas_res_group_reservation_list_item = (
                    componentsschemas_res_group_reservation_list_item_data.to_dict()
                )
                clusters.append(componentsschemas_res_group_reservation_list_item)

        ranges: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.ranges, Unset):
            ranges = []
            for componentsschemas_res_group_range_list_item_data in self.ranges:
                componentsschemas_res_group_range_list_item = (
                    componentsschemas_res_group_range_list_item_data.to_dict()
                )
                ranges.append(componentsschemas_res_group_range_list_item)

        routes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.routes, Unset):
            routes = []
            for componentsschemas_res_group_route_list_item_data in self.routes:
                componentsschemas_res_group_route_list_item = (
                    componentsschemas_res_group_route_list_item_data.to_dict()
                )
                routes.append(componentsschemas_res_group_route_list_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if error is not UNSET:
            field_dict["error"] = error
        if error_code is not UNSET:
            field_dict["errorCode"] = error_code
        if clusters is not UNSET:
            field_dict["clusters"] = clusters
        if ranges is not UNSET:
            field_dict["ranges"] = ranges
        if routes is not UNSET:
            field_dict["routes"] = routes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.res_group_range import ResGroupRange
        from ..models.res_group_reservation import ResGroupReservation
        from ..models.res_group_route import ResGroupRoute

        d = src_dict.copy()

        def _parse_error(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error = _parse_error(d.pop("error", UNSET))

        def _parse_error_code(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        error_code = _parse_error_code(d.pop("errorCode", UNSET))

        clusters = []
        _clusters = d.pop("clusters", UNSET)
        for componentsschemas_res_group_reservation_list_item_data in _clusters or []:
            componentsschemas_res_group_reservation_list_item = (
                ResGroupReservation.from_dict(
                    componentsschemas_res_group_reservation_list_item_data
                )
            )

            clusters.append(componentsschemas_res_group_reservation_list_item)

        ranges = []
        _ranges = d.pop("ranges", UNSET)
        for componentsschemas_res_group_range_list_item_data in _ranges or []:
            componentsschemas_res_group_range_list_item = ResGroupRange.from_dict(
                componentsschemas_res_group_range_list_item_data
            )

            ranges.append(componentsschemas_res_group_range_list_item)

        routes = []
        _routes = d.pop("routes", UNSET)
        for componentsschemas_res_group_route_list_item_data in _routes or []:
            componentsschemas_res_group_route_list_item = ResGroupRoute.from_dict(
                componentsschemas_res_group_route_list_item_data
            )

            routes.append(componentsschemas_res_group_route_list_item)

        res_group_error = cls(
            error=error,
            error_code=error_code,
            clusters=clusters,
            ranges=ranges,
            routes=routes,
        )

        res_group_error.additional_properties = d
        return res_group_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
