import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ResGroupRange")


@_attrs_define
class ResGroupRange:
    """
    Attributes:
        min_freq (float): The start of the frequency range (inclusive) that this grant includes (MHz).
        max_freq (float): The end of the frequency range (inclusive) that this grant includes (MHz).
        resgroup_id (Union[Unset, str]): Unique identifier of the reservation group
        range_id (Union[Unset, str]): Unique identifier of this range reservation
        approved_at (Union[None, Unset, datetime.datetime]): Approval time.
        canceled_at (Union[None, Unset, datetime.datetime]): Cancellation time.
        error (Union[None, Unset, str]): Error message regarding this range
        error_code (Union[None, Unset, int]): Error code regarding this range
    """

    min_freq: float
    max_freq: float
    resgroup_id: Union[Unset, str] = UNSET
    range_id: Union[Unset, str] = UNSET
    approved_at: Union[None, Unset, datetime.datetime] = UNSET
    canceled_at: Union[None, Unset, datetime.datetime] = UNSET
    error: Union[None, Unset, str] = UNSET
    error_code: Union[None, Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)
    _immutable_on = {
        "resgroup_id": {"put": True, "post": True},
        "range_id": {"put": True, "post": True},
        "min_freq": {"put": None, "post": None},
        "max_freq": {"put": None, "post": None},
        "approved_at": {"put": True, "post": True},
        "canceled_at": {"put": True, "post": True},
        "error": {"put": True, "post": True},
        "errorCode": {"put": True, "post": True},
    }

    def to_dict(self) -> Dict[str, Any]:
        min_freq = self.min_freq

        max_freq = self.max_freq

        resgroup_id = self.resgroup_id

        range_id = self.range_id

        approved_at: Union[None, Unset, str]
        if isinstance(self.approved_at, Unset):
            approved_at = UNSET
        elif isinstance(self.approved_at, datetime.datetime):
            approved_at = self.approved_at.isoformat()
        else:
            approved_at = self.approved_at

        canceled_at: Union[None, Unset, str]
        if isinstance(self.canceled_at, Unset):
            canceled_at = UNSET
        elif isinstance(self.canceled_at, datetime.datetime):
            canceled_at = self.canceled_at.isoformat()
        else:
            canceled_at = self.canceled_at

        error: Union[None, Unset, str]
        if isinstance(self.error, Unset):
            error = UNSET
        else:
            error = self.error

        error_code: Union[None, Unset, int]
        if isinstance(self.error_code, Unset):
            error_code = UNSET
        else:
            error_code = self.error_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "min_freq": min_freq,
                "max_freq": max_freq,
            }
        )
        if resgroup_id is not UNSET:
            field_dict["resgroup_id"] = resgroup_id
        if range_id is not UNSET:
            field_dict["range_id"] = range_id
        if approved_at is not UNSET:
            field_dict["approved_at"] = approved_at
        if canceled_at is not UNSET:
            field_dict["canceled_at"] = canceled_at
        if error is not UNSET:
            field_dict["error"] = error
        if error_code is not UNSET:
            field_dict["errorCode"] = error_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        min_freq = d.pop("min_freq")

        max_freq = d.pop("max_freq")

        resgroup_id = d.pop("resgroup_id", UNSET)

        range_id = d.pop("range_id", UNSET)

        def _parse_approved_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                approved_at_type_0 = isoparse(data)

                return approved_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        approved_at = _parse_approved_at(d.pop("approved_at", UNSET))

        def _parse_canceled_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                canceled_at_type_0 = isoparse(data)

                return canceled_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        canceled_at = _parse_canceled_at(d.pop("canceled_at", UNSET))

        def _parse_error(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error = _parse_error(d.pop("error", UNSET))

        def _parse_error_code(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        error_code = _parse_error_code(d.pop("errorCode", UNSET))

        res_group_range = cls(
            min_freq=min_freq,
            max_freq=max_freq,
            resgroup_id=resgroup_id,
            range_id=range_id,
            approved_at=approved_at,
            canceled_at=canceled_at,
            error=error,
            error_code=error_code,
        )

        res_group_range.additional_properties = d
        return res_group_range

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
