import sys
import os
sys.path.append("/usr/testbed/devel/stoller/lib")

import logging
from fastapi import Depends, FastAPI, Header, Request
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from fastapi.middleware.cors import CORSMiddleware

from .routers import experiments
from .routers import resgroups
from .dependencies import get_token_header, PortalException, get_current_user
from .api.models import Error

origins = [
    "http://localhost",
    "http://gitlab.flux.utah.edu"
]

LOG = logging.getLogger("uvicorn.error")
LOG.setLevel(logging.INFO)

app = FastAPI(
    dependencies=[Depends(get_current_user)]
)

@app.exception_handler(PortalException)
def portal_exception_handler(request: Request, exc: PortalException):
    return JSONResponse(
        status_code=exc.code,
        content=Error(error = exc.message).model_dump()
    )

@app.exception_handler(RequestValidationError)
async def validation_exception_handler(request, exc):
    return JSONResponse(
        status_code=400,
        content=Error(error = str(exc)).model_dump()
    )

@app.get("/")
async def root():
    return {"message": "Cloudlab Portal API Server"}

app.include_router(experiments.router)
app.include_router(resgroups.router)
